using NLOptControl, PrettyPlots

# Define problem
prob = define(numStates=2, numControls=1, X0=[0.,1], XF=[0.,-1.], XL=[0.,NaN], XU=[1/9,NaN]);
states!(prob, [:p, :v], descriptions=["p(t)", "v(t)"])
controls!(prob, [:u] ,descriptions=["u(t)"])

# Model
dx = Array{Expr}(2)
dx[1] = :( v[j] ) # dp(j) = v(j)
dx[2] = :( u[j] ) # dv(j) = u(j)
dynamics!(prob,dx)

# Collocation points and free final time
configure!(prob; (:Nck=>[100]),(:finalTimeDV=>true));

# L2 norm objective
obj=integrate!(prob, :(0.5*u[j]^2));
@NLobjective(prob.ocp.mdl, Min, obj);

optimize!(prob);

allPlots(prob)
